# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'CacheMode',
    'CompressionType',
    'Platform',
]


class CacheMode(str, Enum):
    MIN = "min"
    """
    Only layers that are exported into the resulting image are cached.
    """
    MAX = "max"
    """
    All layers are cached, even those of intermediate steps.
    """


class CompressionType(str, Enum):
    GZIP = "gzip"
    """
    Use `gzip` for compression.
    """
    ESTARGZ = "estargz"
    """
    Use `estargz` for compression.
    """
    ZSTD = "zstd"
    """
    Use `zstd` for compression.
    """


class Platform(str, Enum):
    DARWIN_386 = "darwin/386"
    DARWIN_AMD64 = "darwin/amd64"
    DARWIN_ARM = "darwin/arm"
    DARWIN_ARM64 = "darwin/arm64"
    DRAGONFLY_AMD64 = "dragonfly/amd64"
    FREEBSD_386 = "freebsd/386"
    FREEBSD_AMD64 = "freebsd/amd64"
    FREEBSD_ARM = "freebsd/arm"
    LINUX_386 = "linux/386"
    LINUX_AMD64 = "linux/amd64"
    LINUX_ARM = "linux/arm"
    LINUX_ARM64 = "linux/arm64"
    LINUX_MIPS64 = "linux/mips64"
    LINUX_MIPS64LE = "linux/mips64le"
    LINUX_PPC64LE = "linux/ppc64le"
    LINUX_RISCV64 = "linux/riscv64"
    LINUX_S390X = "linux/s390x"
    NETBSD_386 = "netbsd/386"
    NETBSD_AMD64 = "netbsd/amd64"
    NETBSD_ARM = "netbsd/arm"
    OPENBSD_386 = "openbsd/386"
    OPENBSD_AMD64 = "openbsd/amd64"
    OPENBSD_ARM = "openbsd/arm"
    PLAN9_386 = "plan9/386"
    PLAN9_AMD64 = "plan9/amd64"
    SOLARIS_AMD64 = "solaris/amd64"
    WINDOWS_386 = "windows/386"
    WINDOWS_AMD64 = "windows/amd64"
